package net.ligent.students.transportation;

import static org.junit.Assert.*;

import org.junit.Test;

public class BoatTest {

	@Test
	public void testnoArgumentConstructor() {
		Boat b1 = new Boat();
		assertNotNull(b1);
	}

	@Test
	public void testFullyOverloadedConstructor() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		assertEquals(expectedMake, b1.getMake());
		assertEquals(expectedModel, b1.getModel());
		assertEquals(expectedYear, b1.getYear());
		assertEquals(expectedColor, b1.getColor());
		assertEquals(expectedVehicleIdNumber, b1.getVehicleIdNumber());
	}

	@Test
	public void testOverloadedConstructorDoesNotTakeVIN() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength);
		assertEquals(expectedMake, b1.getMake());
		assertEquals(expectedModel, b1.getModel());
		assertEquals(expectedYear, b1.getYear());
		assertEquals(expectedColor, b1.getColor());
		assertNull(b1.getVehicleIdNumber());
	}

	@Test
	public void testToString() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		String expected = "Boat [make=Chris Craft, model=Custom Runabout, year=1946, color=Natural Wood, length=20, vehicleIdNumber=B1234567]";
		String actual = b1.toString();
		assertEquals(expected, actual);
	}

	@Test
	public void testEqualsAllFieldsSame() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		assertEquals(b1, b2);

	}

	@Test
	public void testEqualsVINDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength, "B0987654");
		assertEquals(b1, b2);
	}

	@Test
	public void testEqualsMakeDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat("Hatteras", expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		assertFalse(b1.equals(b2));
	}

	@Test
	public void testEqualsModelDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, "Launch", expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		assertFalse(b1.equals(b2));
	}

	@Test
	public void testEqualsYearDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, expectedModel, 1960, expectedColor, expectedLength, expectedVehicleIdNumber);
		assertFalse(b1.equals(b2));
	}
	
	@Test
	public void testEqualsColorDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, expectedModel, expectedYear, "Midnight Blue", expectedLength, expectedVehicleIdNumber);
		assertFalse(b1.equals(b2));
	}
	
	@Test
	public void testEqualsLengthDifferent() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 20;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		Boat b2 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, 30, expectedVehicleIdNumber);
		assertFalse(b1.equals(b2));
	}

	@Test
	public void testDetermineAnnualRegistrationFeeLength15() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 15;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 28.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}
	
	@Test
	public void testDetermineAnnualRegistrationFeeLength16() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 16;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 60.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}	
	
	@Test
	public void testDetermineAnnualRegistrationFeeLength26() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 26;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 60.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}	
	
	@Test
	public void testDetermineAnnualRegistrationFeeLength27() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 27;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 160.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}		
	
	@Test
	public void testDetermineAnnualRegistrationFeeLength40() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 40;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 160.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}	
	
	@Test
	public void testDetermineAnnualRegistrationFeeLength41() {
		String expectedMake = "Chris Craft";
		String expectedModel = "Custom Runabout";
		int expectedYear = 1946;
		String expectedColor = "Natural Wood";
		int expectedLength = 41;
		String expectedVehicleIdNumber = "B1234567";
		Boat b1 = new Boat(expectedMake, expectedModel, expectedYear, expectedColor, expectedLength,
				expectedVehicleIdNumber);
		double expectedFee = 210.00;
		double actualFee = b1.determineAnnualRegistrationFee();
		assertEquals(expectedFee, actualFee, 0.005);
	}	

}
