package net.ligent.students.transportation;

import static org.junit.Assert.*;

import org.junit.Test;

public class CarTest {

	@Test
	public void testNoArgumentConstructor() {
		Vehicle c1 = new Car();
		assertNotNull(c1);
	}
	
	@Test
	public void testOverloadedConstructor() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Car c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		assertNotNull(c1);
		assertEquals(expectedMake, c1.getMake());
		assertEquals(expectedModel, c1.getModel());
		assertEquals(expectedYear, c1.getYear());
		assertEquals(expectedColor, c1.getColor());
		assertEquals(expectedFuelType, c1.getFuelType());
		assertEquals(expectedVIN, c1.getVehicleIdNumber());				
	}
	
	@Test
	public void testOverloadedConstructorHasNoVIN() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = null;
		Car c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType);
		assertNotNull(c1);
		assertEquals(expectedMake, c1.getMake());
		assertEquals(expectedModel, c1.getModel());
		assertEquals(expectedYear, c1.getYear());
		assertEquals(expectedColor, c1.getColor());
		assertEquals(expectedFuelType, c1.getFuelType());
		assertEquals(expectedVIN, c1.getVehicleIdNumber());				
	}
	
	@Test
	public void testToString() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		String expectedToString = "Car [fuelType=Gas, Vehicle [make=Ford, model=Mustang, year=1966, color=Green, vehicleIdNumber=X1234567]]";
		assertEquals(expectedToString, c1.toString());
	}
	
	@Test
	public void testEqualsPositiveCase() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, "Y0000043");
		assertTrue(c1.equals(c2));
	}
		
	@Test
	public void testEqualsDifferentMake() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car("Chevy", expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		assertFalse(c1.equals(c2));
	}
	
	
	@Test
	public void testEqualsDifferentModel() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car(expectedMake, "Fairlane", expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		assertFalse(c1.equals(c2));
	}
	
	@Test
	public void testEqualsDifferentYear() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car(expectedMake, expectedModel, 1956, expectedColor,
				expectedFuelType, expectedVIN);
		assertFalse(c1.equals(c2));
	}
	
	
	@Test
	public void testEqualsDifferentColor() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car(expectedMake, expectedModel, expectedYear, "Red",
				expectedFuelType, expectedVIN);
		assertFalse(c1.equals(c2));
	}	
	
	@Test
	public void testEqualsDifferentFuelType() {
		String expectedMake = "Ford";
		String expectedModel = "Mustang";
		int expectedYear = 1966;
		String expectedColor = "Green";
		String expectedFuelType = "Gas";
		String expectedVIN = "X1234567";
		Vehicle c1 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				expectedFuelType, expectedVIN);
		Vehicle c2 = new Car(expectedMake, expectedModel, expectedYear, expectedColor,
				"Electric", expectedVIN);
		assertFalse(c1.equals(c2));
	}
	
	@Test
	public void determineAnnualRegistrationFeeElectric() {
		Car c6 = new Car ("Nissan", "Leaf", 2016, "Green", "Electric", "X5544");
		double expected = 50.00;
		double actual= c6.determineAnnualRegistrationFee();
		assertEquals(expected, actual, 0.005);
	}
	
	@Test
	public void determineAnnualRegistrationFeeGas() {
		Car c7 = new Car ("Nissan", "Altima", 2016, "Blue", "Gas", "X3322");
		double expected = 100.00;
		double actual= c7.determineAnnualRegistrationFee();
		assertEquals(expected, actual, 0.005);
	}
	
	@Test
	public void determineAnnualRegistrationFeeDiesel() {
		Car c8 = new Car ("Volkswagon", "Golf", 2015, "Black", "Diesel", "X8888");
		double expected = 150.00;
		double actual= c8.determineAnnualRegistrationFee();
		assertEquals(expected, actual, 0.005);
	}
	
	@Test
	public void determineAnnualRegistrationFeeOther() {
		Car c9 = new Car ("Volkswagon", "Golf", 2015, "Black", "Other", "X7777");
		double expected = 150.00;
		double actual= c9.determineAnnualRegistrationFee();
		assertEquals(expected, actual, 0.005);
	}
	
	@Test
	public void determineAnnualRegistrationFeeNullFuelType() {
		Car c10 = new Car ("Volkswagon", "Golf", 2015, "Black", null, "X6666");
		double expected = 150.00;
		double actual= c10.determineAnnualRegistrationFee();
		assertEquals(expected, actual, 0.005);
	}
	
}
